#!/usr/bin/python 
#-*- coding: UTF-8 -*-
# chordgenerator.py: Generating tables of chords associated with scales.
#
# Copyright (c) 2008-2013 Peter Murphy <peterkmurphy@gmail.com>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * The names of its contributors may not be used to endorse or promote 
#       products derived from this software without specific prior written 
#       permission.
#
# THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ''AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# This software was originally written by Peter Murphy (2008). It has been
# updated in 2011 to use the assoicated musictheory classes. Updated for 2011.
#
# The goal of this utility is to find all the possible chords (major, minor, 
# 7th) in common diatonic scales (such as Major and Harmonic Minor).  The
# results are tables that can be displayed in an HTML file.

import copy, codecs
from musutility import seqtostr;
from temperament import WestTemp, temperament, WestTemp, seq_dict,\
    NSEQ_SCALE, NSEQ_CHORD, M_SHARP, M_FLAT;
from scales import MajorScale, MelMinorScale, HarmMinorScale,\
    HarmMajorScale;
from chords import CHORDTYPE_DICT;

# Used to give a full list of chord types.

CHORDTYPE_ARRAY = ["Fifth", "Triad", "Seventh", "Ninth", "Eleventh", "Thirteenth",
    "Added Ninth", "Suspended", "Suspended Seventh", "Sixth", "Sixth/Ninth",
    "Added Eleventh"];

# Roman numerals are used for the table headers.

# Following routine comes from "Roman Numerals (Python recipe)".
# Author: Paul Winkler on Sun, 14 Oct 2001. See:
# http://code.activestate.com/recipes/81611-roman-numerals/

def int_to_roman(input):
   """
   Convert an integer to Roman numerals.

   Examples:
   >>> int_to_roman(0)
   Traceback (most recent call last):
   ValueError: Argument must be between 1 and 3999

   >>> int_to_roman(-1)
   Traceback (most recent call last):
   ValueError: Argument must be between 1 and 3999

   >>> int_to_roman(1.5)
   Traceback (most recent call last):
   TypeError: expected integer, got <type 'float'>

   >>> for i in range(1, 21): print int_to_roman(i)
   ...
   I
   II
   III
   IV
   V
   VI
   VII
   VIII
   IX
   X
   XI
   XII
   XIII
   XIV
   XV
   XVI
   XVII
   XVIII
   XIX
   XX
   >>> print int_to_roman(2000)
   MM
   >>> print int_to_roman(1999)
   MCMXCIX
   """
   if type(input) != type(1):
      raise TypeError, "expected integer, got %s" % type(input)
   if not 0 < input < 4000:
      raise ValueError, "Argument must be between 1 and 3999"   
   ints = (1000, 900, 500, 400, 100, 90, 50, 40, 10, 9, 5, 4, 1)
   nums = ('M',  'CM', 'D', 'CD','C', 'XC','L','XL','X','IX','V','IV','I')
   result = ""
   for i in range(len(ints)):
      count = int(input / ints[i])
      result += nums[i] * count
      input -= ints[i] * count
   return result

def make_roman_numeral_list(num):
    """ Make a sequence of roman numerals from 1 to num. """
    return([int_to_roman(i) for i in range(1, num + 1)]);

# In Django, do we print out abbreviations or full chord names?

PRINT_ABBRV = 0;
PRINT_FNAME = 1;
PRINT_BOTH = 2;

class scale_chords():
    """ Represents all the chords associated with a scale. This is
        used to make a tabular representation.
    """
    def __init__ (self, full_name, key, full_notes, table_title):
        self.full_name = full_name;
        self.key = key;
        self.full_notes = full_notes;
        self.table_title = table_title;
        self.rows = [];

class scale_chordrow():
    """ Represents different rows (triads, 7ths, 9ths) in tabular
        representations of scale chords.
    """
    def __init__ (self, chord_type):
        self.chord_type = chord_type;
        self.notes = [];
        
class scale_chordcell():
    """ Represents different cells in the table of chords. """
    def __init__ (self, chordname_1, chordname_2, notes):
        self.chordname_1 = chordname_1;
        self.chordname_2 = chordname_2;
        self.notes = notes;

def makebaserep(notex, base = 0):
    """ Used for converting "C" -> 1, "Db"-> 2b, etc. """
    notexparsed = WestTemp.note_parse(notex);
    pos_rep = unicode(WestTemp.nat_key_lookup_order[notexparsed[0]] + base);
    if notexparsed[1] > 0:
        return pos_rep + (M_SHARP * notexparsed[1]);
    elif notexparsed[1] < 0:
        return pos_rep + (M_FLAT * (-1 * notexparsed[1]));
    else:
        return pos_rep;

def populate_scale_chords(scale_name, key, possiblechords):
    """ Returns an instance of scale_chords using the following inputs:

        scale_name: a name of a scale like "Dorian".
        key: generally a standard music key like "C".
        possiblechords: a sequence of chord types like "Seventh" and "Ninth".
    """
    our_scale = WestTemp.get_nseqby_name(scale_name, NSEQ_SCALE);
    num_elem = len(our_scale.nseq_posn);
    try: 
        int_of_key = int(key)
        is_key_an_int = True;
    except ValueError:
        is_key_an_int = False;
        int_of_key = None;
    if is_key_an_int:
        our_scale_notes = [makebaserep(x, int_of_key) for x in 
            our_scale.get_notes_for_key("C")];
    else:
        our_scale_notes = our_scale.get_notes_for_key(key); 
    our_chord_data = scale_chords(scale_name, key, our_scale_notes,
        make_roman_numeral_list(num_elem));
    for i in possiblechords:
        ourchordrow = scale_chordrow(i);
        our_chord_data.rows.append(ourchordrow);
        for j in range(num_elem):
            our_slice = CHORDTYPE_DICT[i];
            if is_key_an_int:
                our_chord_notes = [makebaserep(x, int_of_key) for x in
                    our_scale.get_notes_for_key("C", j, our_slice)];
            else:
                our_chord_notes = our_scale.get_notes_for_key(key, j, 
                    our_slice); 
            our_posn = our_scale.get_posn_for_offset(j, our_slice, True);
            our_chord = WestTemp.get_nseqby_seqpos(our_posn, NSEQ_CHORD);
            if our_chord:
                ourchordrow.notes.append(scale_chordcell(our_chord.nseq_name,
                    our_chord.nseq_abbrev, our_chord_notes));
            else:
                ourchordrow.notes.append(scale_chordcell("", "", 
                    our_chord_notes));
    return our_chord_data;    

def chordgentable(scales):
    """ Generates a table representation of a series of scales,
        represented by a scale_chords instance. """
    startrow  = "<tr>\n";
    endrow = "</tr>\n";
    thestring = u"";
    for scale in scales:
        thestring += "<p><h2>%s</h2></p>\n" % (scale.key + " " +scale.full_name);
        thestring += ("<table id=\"%s\" class=\"chordtable\">\n" % 
            (scale.key + " " +scale.full_name));
        thestring += "<caption>%s</caption>\n" % (scale.key + " " + scale.full_name +
            ": " + seqtostr(scale.full_notes));
        thestring += "<thead>\n" + startrow + "<th>Chord Types</th>\n";
        for q in range(7):
            thestring += "<th>%s</th>\n" % str(int_to_roman(q+1));
        thestring += endrow + "</thead>\n<tbody>\n";

        for i in scale.rows:
            thestring += startrow;    
            thestring += "<td>%s</td>\n" % i.chord_type;
            for j in i.notes:
                if not j.chordname_1:
                    thestring += ("<td><p>%s<br />" % (unicode(j.chordname_1)));
                    thestring += ("<i>%s</i><br />" % (unicode(j.chordname_2)));
                else:
                    thestring += ("<td><p>%s<br />" % (j.notes[0]+" "+unicode(j.chordname_1)));
                    thestring += ("<i>%s</i><br />" % (j.notes[0]+unicode(j.chordname_2)));
                    
                thestring += ("<b>%s</b></p></td>" % seqtostr(j.notes));                
            thestring += endrow;    
        thestring += "\n</tbody>\n</table>\n";    
    return thestring;

if __name__ == "__main__":

    def xhtmlchordgencontent(filename, content):
        """ Generates an XHTML file using the output of chordgenerator. This
        function is called only when executing this module as __main__. """
        ffilename = codecs.open(filename, "w", "utf-8");
        bplate = """<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" 
        "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
        <html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
        <head>\n<meta http-equiv="Content-Type" content="text/html; 
        charset=utf-8" />\n<title>Chord Generator</title>
        <style type="text/css" media="screen"> h1 {color: blue;
        text-align: center;} table {border: 1px solid #000000; 
        border-collapse: collapse;} td {border: 1px solid #000000;}
        th {border: 1px solid #000000;}</style>\n
        </head><body>\n<h1>Chord Instances</h1>\n%s"</body>\n</html>\n""";
        output = bplate % content;
        ffilename.write(output);
        ffilename.close();

    MajorStuff = populate_scale_chords("Major", u"C♯", CHORDTYPE_ARRAY); 
    MelMinorStuff = populate_scale_chords("Melodic Minor", u"C♯", 
        CHORDTYPE_ARRAY); 
    HarmMinStuff = populate_scale_chords("Harmonic Minor", u"C♯", 
        CHORDTYPE_ARRAY); 
    HarmMajStuff = populate_scale_chords("Harmonic Major", u"C♯", 
        CHORDTYPE_ARRAY); 

    tabledata = chordgentable([MajorStuff, MelMinorStuff, HarmMinStuff,
        HarmMajStuff]);
    xhtmlchordgencontent("chordgenerator.html", tabledata);
    print WestTemp.seq_maps.nseqtype_maps[NSEQ_SCALE].name_dict.keys()



#!/usr/bin/python 
#-*- coding: UTF-8 -*-
# musutility.py: Defines utility functions not properly part of other modules.
#
# Copyright (c) 2008-2013 Peter Murphy <peterkmurphy@gmail.com>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * The names of its contributors may not be used to endorse or promote 
#       products derived from this software without specific prior written 
#       permission.
#
# THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ''AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import sys;
import codecs;
import unittest;

# One of the common problems with python 2.x is that its print function does 
# not handle Unicode strings with characters values greater than 127. For this
# reason, we provide our own. It is used only for debugging. 

def printunicode(value, encoding = "utf-8"):
    """ Prints an unescaped unicode string to the system console. """
    uval = unicode(value);
    sys.stdout.write(codecs.encode(uval + '\n', encoding));

# Make sequences representable as strings.

def seqtostr(value):
    """ Writes a sequence as a comma delimited string. """
    output = "";
    if not value:
        return output;
    for i in value:
        output += unicode(i) + ", ";
        
# Trim the last two characters.

    return output[:-2];


# The next few functions are for rotating sequences and choosing "multiple
# element slices" See multislice for more information.

def rotate(seq, offset):
    """ Rotates a seq to the left by offset places; the elements shifted off
    are inserted back at the end. By setting offset to the negative number -N,
    this function rotates the sequence N places.
    
    Note: this algorithm is a modification of one originally provided by
    Thorsten Kampe; this version handles zero sequences and right shifts. See:
    http://bytes.com/topic/python/answers/36070-rotating-lists
    """
    if (len(seq) == 0) or (offset == 0):
        return seq;
    else:
        offsetmod = offset % len(seq);
        return seq[offsetmod:] + seq[:offsetmod];

def scalar_addition(seq, scalar, mod = 0):
    """ Returns the result of adding scalar (a number) to all elements in seq.
        If mod is not 0, then all addition occurs modulo mod. 
    
        Note: this function is an "analogue" of scalar multiplication - an 
        operation where every element in a sequence is multiplied by a scalar.
        We can also use this function to perform "scalar subtraction" - the
        subtraction of a scalar from all elements in a sequence. 
    """
    if mod == 0:
        return [(index + scalar) for index in seq];
    else:
        return [((index + scalar) % mod) for index in seq];

def rotate_and_zero(seq, offset, mod = 0):
    """ Returns the result of:
        (a) rotating a sequence to a given offset; and then,
        (b) scalar subtracting the resulting first element from the rotated
            sequence.
        The result always begins with zero (except when seq is []).
        If mod is not 0, then all subtraction occurs modulo mod.
        
        Note: this function is useful in generating different modes of a scale
        and different synonyms of a chord.
    """
    if len(seq) == 0:
        return seq;
    rotated_seq = rotate(seq, offset);
    first_elem = rotated_seq[0];
    return scalar_addition(rotated_seq, 0 - first_elem, mod);

def multislice(seq, slice, mod = 0, offset = 0):
    """ The multislice function takes a multiple-element-slice (see below) of
        a sequence, rotates it if necessary afterward, and returns it. The
        arguments:
        seq: a sequence to slice and/or rotate.
        slice: another sequence which specifies the elements preserved in seq.
            This consists of integers, which indicate the indices of elements
            in seq to preserve. If i is in slice, then seq[i] is
            preserved; otherwise, it is discarded. (Slicing always happens
            before rotation).
        mod: if present, then all indices in slice are taken modulo this
            number. I.e., seq[j] is preserved if and only if j some k modulo
            mod, where k is in slice.
        offset: after slicing, this sequence is rotated this number of places.
    """
    if mod == 0:
        return [rotate(seq, offset)[index] for index in slice];
    else:
        return [rotate(seq, offset)[index % mod] for index in slice];

def repseq(seq, pre_process = lambda x: x):
    """ Generates a unicode string from the items in seq delimited by commas.
        For example repseq([1, 2, 3]) produces "1, 2, 3". Arguments:
        seq: a sequence of items.
        pre_process: a function taking one argument and returning another. The
            function will execute this on each item in seq before concatenating
            them together. By default, nothing is done to each argument.
    """
    SPACECOMMA = ", ";
    if len(seq) == 0:
        return u"";
    str_out = ''.join([(unicode(pre_process(item)) + SPACECOMMA) 
        for item in seq[:-1]]) + unicode(pre_process(seq[-1]));
    return str_out;

def enl_seq(seq_of_seq, otherentries):
    """ Returns a sequence of sequences, with each element consisting of an
        element from otherentries appended to a sequence from seq_of_seq.

        Note: this is used for building chords.
    """
    ret = [];
    for i in seq_of_seq:
        for j in otherentries:
            ret.append(i + [j]);
    return ret;

def norm_seq(seq, mod):
    """ This function takes a sequence (of numbers) and "normalizes" it. To  
        be specific, it evaluates all numbers inside seq modulo mod. The
        result is then sorted, and any duplicates are removed. 
        
        Note: this function is useful for looking up chords by their patterns.
    """
    sorted_seq = sorted(map(lambda x: x % mod, seq));

# We remove duplicates. The following code is straight outta the Python FAQ.
    
    last = sorted_seq[-1]
    for i in range(len(sorted_seq)-2, -1, -1):
        if last == sorted_seq[i]:
            del sorted_seq[i]
        else:
            last = sorted_seq[i]
    return sorted_seq;


if __name__ == "__main__":
    
    class TestMusutility(unittest.TestCase):
        """ This only tests functions in this module. """
        
        def setUp(self):
            self.eseq = [];
            self.one_elem_seq = ["0"];
            self.one_elem_seq_one = ["1"]
            self.two_elem_seq = ["0", "1"];
            self.two_elem_seq_rot = ["1", "0"];
            self.three_elem_seq = ["0", "1", "2", "3"];
            self.three_elem_seq_left = ["1", "2", "3", "0"];
            self.three_elem_seq_right = ["3", "0", "1", "2"]; 
            self.all_test_seq = [[], ["0"], ["0", "1"], ["1", "0"],
                ["0", "1", "2", "3"], ["1", "2", "3", "0"], 
                ["3", "0", "1", "2"]];

        def test_rotate(self):
            self.assertEqual(rotate(self.eseq, 0), self.eseq);
            self.assertEqual(rotate(self.eseq, 1), self.eseq);
            self.assertEqual(rotate(self.eseq, -1), self.eseq);
            self.assertEqual(rotate(self.one_elem_seq, 0), 
                self.one_elem_seq);
            self.assertEqual(rotate(self.one_elem_seq, 1), 
                self.one_elem_seq);
            self.assertEqual(rotate(self.one_elem_seq, -1), 
                self.one_elem_seq);
            self.assertEqual(rotate(self.two_elem_seq, 0), 
                self.two_elem_seq);
            self.assertEqual(rotate(self.two_elem_seq, 1), 
                self.two_elem_seq_rot);
            self.assertEqual(rotate(self.two_elem_seq, -1), 
                self.two_elem_seq_rot);
            self.assertEqual(rotate(self.three_elem_seq, 0), 
                self.three_elem_seq);
            self.assertEqual(rotate(self.three_elem_seq, 1), 
                self.three_elem_seq_left);
            self.assertEqual(rotate(self.three_elem_seq, -1), 
                self.three_elem_seq_right);
 
        def test_scalar_addition(self):
            
# Tests on small sequences.            
            
            self.assertEqual(scalar_addition([], 0, 0), []);
            self.assertEqual(scalar_addition([], 0, 5), []);
            self.assertEqual(scalar_addition([], 5, 0), []);
            self.assertEqual(scalar_addition([], 5, 5), []);
            self.assertEqual(scalar_addition([1], 0, 0), [1]);
            self.assertEqual(scalar_addition([1], 0, 5), [1]);
            self.assertEqual(scalar_addition([1], 5, 0), [6]);
            self.assertEqual(scalar_addition([1], 5, 5), [1]);            
            self.assertEqual(scalar_addition([-1], 0, 0), [-1]);
            self.assertEqual(scalar_addition([-1], 0, 5), [4]);
            self.assertEqual(scalar_addition([-1], 5, 0), [4]);
            self.assertEqual(scalar_addition([-1], 5, 5), [4]);  

# Tests on sequences used in the representation of the major chord.

            self.assertEqual(scalar_addition([4, 7, 0], -4, 12), [0, 3, 8]);
            self.assertEqual(scalar_addition([3, 8, 0], -3, 12), [0, 5, 9]);
            self.assertEqual(scalar_addition([5, 9, 0], -5, 12), [0, 4, 7]);
            self.assertEqual(scalar_addition([2, 4, 0], -2, 7), [0, 2, 5]);
            self.assertEqual(scalar_addition([2, 5, 0], -2, 7), [0, 3, 5]);
            self.assertEqual(scalar_addition([3, 5, 0], -3, 7), [0, 2, 4]);

        def test_rotate_and_zero(self):

# Tests on small sequences.            
            
            self.assertEqual(rotate_and_zero([], 0, 0), []);
            self.assertEqual(rotate_and_zero([], 0, 5), []);
            self.assertEqual(rotate_and_zero([], 5, 0), []);
            self.assertEqual(rotate_and_zero([], 5, 5), []);
            self.assertEqual(rotate_and_zero([1], 0, 0), [0]);
            self.assertEqual(rotate_and_zero([1], 0, 5), [0]);
            self.assertEqual(rotate_and_zero([1], 5, 0), [0]);
            self.assertEqual(rotate_and_zero([1], 5, 5), [0]);            
            self.assertEqual(rotate_and_zero([-1], 0, 0), [0]);
            self.assertEqual(rotate_and_zero([-1], 0, 5), [0]);
            self.assertEqual(rotate_and_zero([-1], 5, 0), [0]);
            self.assertEqual(rotate_and_zero([-1], 5, 5), [0]);  

# Tests on sequences used in the representation of the major chord.

            self.assertEqual(rotate_and_zero([0, 4, 7], 1, 12), [0, 3, 8]);
            self.assertEqual(rotate_and_zero([0, 3, 8], 1, 12), [0, 5, 9]);
            self.assertEqual(rotate_and_zero([0, 5, 9], 1, 12), [0, 4, 7]);
            self.assertEqual(rotate_and_zero([0, 2, 4], 1, 7), [0, 2, 5]);
            self.assertEqual(rotate_and_zero([0, 2, 5], 1, 7), [0, 3, 5]);
            self.assertEqual(rotate_and_zero([0, 3, 5], 1, 7), [0, 2, 4]);

# And one for the diminished chord.

            self.assertEqual(rotate_and_zero([0, 3, 6, 9], 1, 12), 
                [0, 3, 6, 9]);

        def test_multislice(self):
            zero_slice = [0];
            one_slice = [1];
            zero_and_one_slice = [0, 1];

# For all sequences, the empty slice ([]) acting on it returns [], irrespective
# of the modulo and the offset.

            for i in range(8):
                for j in range(4):
                    for k in self.all_test_seq:
                        self.assertEqual(multislice(k, self.eseq,
                            i, j), self.eseq);

# For all sequences of one element, the zero slice ([0]) returns the same 
# sequence, irrespective of the modulo and the offset.

            for i in range(4):
                for j in range(4):
                    self.assertEqual(multislice(self.one_elem_seq,
                        zero_slice, i, j), self.one_elem_seq);

# Now we try slices on two elements.

            for j in range(4):
                for i in [0, 2, 4, 6]:
                    self.assertEqual(multislice(self.two_elem_seq, 
                        zero_slice, j, i), self.one_elem_seq);
                    self.assertEqual(multislice(self.two_elem_seq_rot, 
                        zero_slice, j, i), self.one_elem_seq_one);                    
                for i in [1, 3, 5, 7]:
                    self.assertEqual(multislice(self.two_elem_seq, 
                        zero_slice, j, i), self.one_elem_seq_one);
                    self.assertEqual(multislice(self.two_elem_seq_rot, 
                        zero_slice, j, i), self.one_elem_seq); 

# Slices of three elements should be tested later, but I would rather look at
# implementing other routines. I may come back to them, or not.

        def test_repseq(self):
            doubler = lambda x: x * 2;
            self.assertEqual(repseq(self.eseq), "");
            self.assertEqual(repseq(self.eseq, doubler), "");
            self.assertEqual(repseq([1]), "1");
            self.assertEqual(repseq([1], doubler), "2");
            self.assertEqual(repseq([1, 2]), "1, 2");
            self.assertEqual(repseq([1, 2], doubler), "2, 4");            

        def test_enl_seq(self):
            self.assertEqual(enl_seq(self.eseq, self.eseq), self.eseq);
            self.assertEqual(enl_seq(self.eseq, self.one_elem_seq), self.eseq);
            self.assertEqual(enl_seq(self.one_elem_seq, self.eseq), self.eseq);
            self.assertEqual(enl_seq([["0"]], self.one_elem_seq_one),
                [["0","1"]]);

        def test_norm_seq(self):
            self.assertEqual(norm_seq([7, 6, 5], 7), [0, 5, 6]);
            self.assertEqual(norm_seq([5, 6, 7], 7), [0, 5, 6]);
            self.assertEqual(norm_seq([7, 6, 5], 12), [5, 6, 7]);
            self.assertEqual(norm_seq([5, 6, 7], 12), [5, 6, 7]);
            
    unittest.main()
   
    
    




#!/usr/bin/python 
#-*- coding: UTF-8 -*-
# __init__.py: Defines the module for musictheory.
#
# Copyright (c) 2008-2013 Peter Murphy <peterkmurphy@gmail.com>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * The names of its contributors may not be used to endorse or promote 
#       products derived from this software without specific prior written 
#       permission.
#
# THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ''AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

__all__ = ["musutility", "temperament", "scales", "chords"];
__author__ = "Peter Murphy (peterkmurphy@gmail.com)"
__version__ = "0.4"
__copyright__ = "Copyright (c) 2008-2013 Peter Murphy"
__license__ = "BSD (3 clause)"
    



#!/usr/bin/python 
#-*- coding: UTF-8 -*-
# scales.py: Defines (and contains) the classes for chords.
#
# Copyright (c) 2008-2013 Peter Murphy <peterkmurphy@gmail.com>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * The names of its contributors may not be used to endorse or promote 
#       products derived from this software without specific prior written 
#       permission.
#
# THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ''AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import unittest;

from musutility import enl_seq;
from temperament import temperament, WestTemp, seq_dict, NSEQ_SCALE, \
    NSEQ_CHORD, M_SHARP, M_FLAT;
from scales import noteseq, noteseq_scale;    

class noteseq_chord(noteseq):
    """ A specialisation of noteseq used exclusively for defining chords -
        especially chords associated with heptatonic/diotonic scales.
    """
    def __init__(self, nseq_name, nseq_temp, nseq_posn, nseq_nat_posns, 
        nseq_abbrev):
        """ Arguments:
            nseq_name: name of chord.
            nseq_temp: temperament for chord.
            nseq_posn: position of notes in chord.
            nseq_nat_posns: natural note positions for notes in scales.
            nseq_abbrev: main abbreviation for chord.
        """
        noteseq.__init__(self, nseq_name, NSEQ_CHORD, nseq_temp, nseq_posn, 
            nseq_nat_posns, nseq_abbrev); 

    def __unicode__(self):
        return unicode(self.nseq_name)+":"+unicode(self.nseq_abbrev)+":" \
            + unicode(self.nseq_posn);

# This dictionary contains a list of different nseq_nat_posns values for 
# different types of chords in the Western tradition. Note that all arrays are
# stored base 0, not base1, so a "seventh" chord is stored as [0, 2, 4, 6] -
# not [1, 3, 5, 7]. 

CHORDTYPE_DICT = {"Triad": [0, 2, 4],
    "Seventh": [0, 2, 4, 6],
    "Ninth": [0, 2, 4, 6, 8],
    "Eleventh": [0, 2, 4, 6, 8, 10],
    "Thirteenth": [0, 2, 4, 6, 8, 10, 12],
    "Added Ninth": [0, 2, 4, 8],
    "Suspended": [0, 3, 4],
    "Suspended Seventh": [0, 3, 4, 6],
    "Sixth": [0, 2, 4, 5],
    "Sixth/Ninth": [0, 2, 4, 5, 8],
    "Added Eleventh": [0, 2, 4, 10],
    "Fifth": [0, 4]};


def generate_west_chords():
    """ This function generates most of the chords in the Western tradition
        (and some that are not really chords at all).
    """
    chordseq = [];
    bases = [[0, 3], [0, 4]];
    triads = enl_seq(bases, [6, 7, 8]);
    suspendeds = [[0, 5, 6], [0, 5, 7], [0, 5, 8]];
    sixths = enl_seq(triads, [8, 9]);
    sevenths = enl_seq(triads, [10, 11]);
    suspended_sevenths = enl_seq(suspendeds, [10, 11]);
    ninths = enl_seq(sevenths, [13, 14, 15]);
    elevenths = enl_seq(ninths, [17, 18]);
    thirteenths = enl_seq(elevenths, [20, 21]);
    add_ninths = enl_seq(triads, [13, 14, 15]);
    six_ninths = enl_seq(sixths, [13, 14, 15]);
    sevenths_and_above = sevenths + ninths + elevenths + thirteenths + \
        suspended_sevenths;

# We add the power chords.

    chordseq.append(noteseq_chord("Power Fifth", WestTemp, [0, 7], 
        CHORDTYPE_DICT["Fifth"], "5"));
    chordseq.append(noteseq_chord("Tritone", WestTemp, [0, 6], 
        CHORDTYPE_DICT["Fifth"], "T"));
    chordseq.append(noteseq_chord("Power Sharp Fifth", WestTemp, [0, 8], 
        CHORDTYPE_DICT["Fifth"], "+5"));


# We add the diminished chords, as they take special handling.

    chordseq.append(noteseq_chord("Diminish 7th", WestTemp, [0, 3, 6, 9], 
        CHORDTYPE_DICT["Seventh"], "dim7"));
    chordseq.append(noteseq_chord("Diminish 9th", WestTemp, [0, 3, 6, 9, 13], 
        CHORDTYPE_DICT["Ninth"], "dim9"));
    chordseq.append(noteseq_chord("Diminish 11th", WestTemp, 
        [0, 3, 6, 9, 13, 16], CHORDTYPE_DICT["Eleventh"], "dim11"));
    chordseq.append(noteseq_chord("Diminish 13th", WestTemp, 
        [0, 3, 6, 9, 13, 16, 20], CHORDTYPE_DICT["Thirteenth"], "dim13"));

# We loop over the triads and the added ninths together. For this reason, we 
# set up some maps for the ease of iteration.

    triad_names = {3: {6:"Diminish 5th", 7:"Minor", 8:"Minor Sharp 5th"}, 
        4: { 6:"Major Flat 5th", 7:"Major", 8:"Augment"}}; 
    triad_abbrv = {3: {6:"dim5", 7:"min", 8:"min+5"}, 
        4: { 6:"-5", 7:"maj", 8:"+"}}; 
    add9_names = {13:" Add Flat 9th", 14: " Add 9th", 
        15: " Add Sharp 9th"};
    add9_abbrv = {13:" add-9", 14: " add9", 15: " add+9"};

    add11_names = {17:" Add 11th", 18: " Add Sharp 11th"};
    add11_abbrv = {17:" add11", 18: " add+11"};

    add6_names = {20: " Add Flat 6th", 21:" Add 6th"};
    add6_abbrv = {20: " add-6", 21:" add6"};


    for i in [3, 4]:
        for j in [6, 7, 8]:
            our_tname = triad_names[i][j];
            our_tabbr = triad_abbrv[i][j];
            chordseq.append(noteseq_chord(our_tname, WestTemp, 
                [0, i, j], CHORDTYPE_DICT["Triad"], our_tabbr));
            for k in [13, 14, 15]:
                if i == 3 and k == 15:
                    pass;
                else:
                    chordseq.append(noteseq_chord(our_tname + add9_names[k],
                        WestTemp, [0, i, j, k], CHORDTYPE_DICT["Added Ninth"],
                        our_tabbr+add9_abbrv[k]));
            for k in [17, 18]:
                if k == 18 and j == 6:
                    pass;
                else:
                    chordseq.append(noteseq_chord(our_tname + add11_names[k],
                        WestTemp, [0, i, j, k], CHORDTYPE_DICT["Added Eleventh"],
                        our_tabbr+add11_abbrv[k]));
            for k in [20, 21]:
                if i == 3 and j == 6 and k == 21:
                    pass; # Pattern already taken by Diminished 7 chord.
                else:
                    chordseq.append(noteseq_chord(our_tname + add6_names[k],
                        WestTemp, [0, i, j, k], CHORDTYPE_DICT["Sixth"],
                        our_tabbr+add6_abbrv[k]));

                for l in [13, 14, 15]:
                    if i == 3 and l == 15:
                        pass;
                    else:
                        chordseq.append(noteseq_chord(
                            our_tname + add6_names[k] + add9_names[l],
                            WestTemp, [0, i, j, k, l], 
                            CHORDTYPE_DICT["Sixth/Ninth"],
                            our_tabbr+add6_abbrv[k]+add9_abbrv[l]));


# Then we add the suspended chords.

    chordseq.append(noteseq_chord("Suspend Flat 5th", WestTemp, [0, 5, 6], 
        CHORDTYPE_DICT["Suspended"], "sus-5"));
    chordseq.append(noteseq_chord("Suspend", WestTemp, [0, 5, 7], 
        CHORDTYPE_DICT["Suspended"], "sus"));
    chordseq.append(noteseq_chord("Suspend Sharp 5th", WestTemp, [0, 5, 8], 
        CHORDTYPE_DICT["Suspended"], "sus+5"));

# Afterwards, we add the 7th (including sus 7th), 9th, 11th and 13th chords.     
    
    for i in sevenths_and_above:
        abbrev = "";
        name = "";
        if 5 in i:
            abbrev += "sus / ";
            name += "Suspend "            
        if 3 in i:
            if 11 in i:
                abbrev += "maj / min";
                name +="Major / Minor "
            else:
                abbrev += "min";
                name += "Minor ";
        else:
            if 11 in i:
                abbrev += "maj";
                name += "Major "
        if 21 in i:
            abbrev += "13";
            name += "13th "
        elif 17 in i:
            abbrev += "11";
            name += "11th "
        elif 14 in i:
            abbrev += "9";
            name += "9th "
        else:
            abbrev += "7";
            name += "7th "
        if 20 in i:
            abbrev += "-13";
            name += "Flat 13th "
        if 18 in i:
            abbrev += "+11";
            name += "Sharp 11th "
        if 15 in i:
            abbrev += "+9";
            name += "Sharp 9th "
        if 13 in i:
            abbrev += "-9";
            name += "Flat 9th "
        if 8 in i:
            abbrev += "+5";
            name += "Sharp 5th "
        if 6 in i:
            abbrev += "-5";
            name += "Flat 5th ";
            
# We eliminate chords that consist of the same note seperated by an octave!            
            
        if 3 in i and 15 in i:
            pass;
        elif 5 in i and 17 in i:
            pass;
        elif 6 in i and 18 in i:
            pass;
        elif 8 in i and 20 in i:
            pass;              
        else:
            name = name.rstrip();
            if 5 in i:
                our_chordtype = CHORDTYPE_DICT["Suspended Seventh"];
            elif 20 in i or 21 in i:
                our_chordtype = CHORDTYPE_DICT["Thirteenth"];
            elif 17 in i or 18 in i:
                our_chordtype = CHORDTYPE_DICT["Eleventh"];                
            elif 13 in i or 14 in i or 15 in i:
                our_chordtype = CHORDTYPE_DICT["Ninth"];
            else:
                our_chordtype = CHORDTYPE_DICT["Seventh"];
            chordseq.append(noteseq_chord(name, WestTemp, i, 
                our_chordtype, abbrev));

    return chordseq;
    
ourchords = generate_west_chords();  

if __name__ == "__main__":
    for i in ourchords:
        print unicode(i);

    class TestChords(unittest.TestCase):
        """ This tests the generation of common chords (i.e, major). """
        
        def setUp(self):
            self.majorchord = WestTemp.get_nseqby_name("Major", NSEQ_CHORD);
           
        def test_temperament_dict(self):
            """ Like test_seq_dict, but checks dictionary in WestTemp. """
            self.assertTrue(WestTemp.check_nseqby_subdict(NSEQ_CHORD));
            self.assertTrue(WestTemp.check_nseqby_name(NSEQ_CHORD, "Major"));
            self.assertTrue(WestTemp.check_nseqby_name(NSEQ_CHORD, "Minor"));
            self.assertFalse(WestTemp.check_nseqby_name(NSEQ_CHORD, "Chord"));
            self.assertTrue(WestTemp.check_nseqby_abbrv(NSEQ_CHORD, 
                "maj"));
            self.assertTrue(WestTemp.check_nseqby_abbrv(NSEQ_CHORD, 
                "min"));
            self.assertFalse(WestTemp.check_nseqby_abbrv(NSEQ_SCALE, 
                "chd"));
            self.assertTrue(WestTemp.check_nseqby_seqpos(NSEQ_CHORD, 
                [0, 4, 7]));
            self.assertEqual(WestTemp.get_nseqby_name("Major", NSEQ_CHORD), 
                self.majorchord);

        def test_get_notes_for_chord(self):
            self.assertEqual(self.majorchord.get_notes_for_key("Eb", 
                0, [0]), [u'E\u266d']);            
            self.assertEqual(self.majorchord.get_notes_for_key("Eb", 
                1, [0]), ['G']);            
            self.assertEqual(self.majorchord.get_notes_for_key("Eb", 
                2, [0]), [u'B\u266d']);            
            self.assertEqual(self.majorchord.get_notes_for_key("Eb", 
                3, [0]), [u'E\u266d']);            
            self.assertEqual(self.majorchord.get_notes_for_key("Eb", 
                0, [0, 2]), [u'E\u266d', u'B\u266d']);            
            self.assertEqual(self.majorchord.get_notes_for_key("Eb", 
                1, [0, 2]), ['G', u'E\u266d']);            
            self.assertEqual(self.majorchord.get_notes_for_key("Eb", 
                2, [0, 2]), [u'B\u266d', 'G']);            
            self.assertEqual(self.majorchord.get_notes_for_key("Eb", 
                3, [0, 2]), [u'E\u266d', u'B\u266d']);
            self.assertEqual(self.majorchord.get_notes_for_key("Eb", 
                0, [0, 1]), [u'E\u266d', 'G']);            
            self.assertEqual(self.majorchord.get_notes_for_key("Eb", 
                1, [0, 1]), ['G', u'B\u266d']);            
            self.assertEqual(self.majorchord.get_notes_for_key("Eb", 
                2, [0, 1]), [u'B\u266d', u'E\u266d',]);            
            self.assertEqual(self.majorchord.get_notes_for_key("Eb", 
                3, [0, 1]), [u'E\u266d', 'G']);                


        def test_get_posn_for_chord(self):
            self.assertEqual(self.majorchord.get_posn_for_offset(0, 
                raz=False), [0, 4, 7]);
            self.assertEqual(self.majorchord.get_posn_for_offset(0, 
                raz=True), [0, 4, 7]);            
            self.assertEqual(self.majorchord.get_posn_for_offset(0, [0], 
                raz=False), [0]);
            self.assertEqual(self.majorchord.get_posn_for_offset(0, [0], 
                raz=True), [0]);
            self.assertEqual(self.majorchord.get_posn_for_offset(0, [1], 
                raz=False), [4]);
            self.assertEqual(self.majorchord.get_posn_for_offset(0, [1], 
                raz=True), [0]);
            self.assertEqual(self.majorchord.get_posn_for_offset(0, [2], 
                raz=False), [7]);
            self.assertEqual(self.majorchord.get_posn_for_offset(0, [2], 
                raz=True), [0]);                
            self.assertEqual(self.majorchord.get_posn_for_offset(1, 
                raz=False), [4, 7, 0]);
            self.assertEqual(self.majorchord.get_posn_for_offset(1, 
                raz=True), [0, 3, 8]);            
            self.assertEqual(self.majorchord.get_posn_for_offset(1, [0], 
                raz=False), [4]);
            self.assertEqual(self.majorchord.get_posn_for_offset(1, [0], 
                raz=True), [0]);
            

    unittest.main()







#!/usr/bin/python 
#-*- coding: UTF-8 -*-
# scales.py: Defines the classes for note sequences and scales.
#
# Copyright (c) 2008-2013 Peter Murphy <peterkmurphy@gmail.com>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * The names of its contributors may not be used to endorse or promote 
#       products derived from this software without specific prior written 
#       permission.
#
# THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ''AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import unittest;

from musutility import rotate, rotate_and_zero, multislice;
from temperament import temperament, WestTemp, seq_dict, NSEQ_SCALE, \
    NSEQ_CHORD, M_SHARP, M_FLAT, CHROM_NAT_NOTE_POS;

class noteseq:
    """ This represents a pattern of notes - a scale or a chord - each with a
        specified position from a base key or note. For example, a major scale
        is specified as the pattern [0, 2, 4, 5, 7, 9, 11], while a major chord
        is specified as [0, 4, 7].[*] In each case, we don't specify the key.
        However, this class provides functions that allow users to generate
        notes with a specified key as a function argument.
        
        [* If this doesn't make sense, consider the first note of a major scale
        is 0 semitones from the first note of a major scale, the second note is
        2 semitones (a tone) from the first note, the third note is 4 semitones
        from the first note... and so on to the final note of the scale, which
        is 11 tones from the first note. Exercise for the reader: try this
        concept with a _major_ chord. ]
        """
    def __init__(self, nseq_name, nseq_type, nseq_temp, nseq_posn, 
        nseq_nat_posns, nseq_abbrev = "", nseq_synonyms = [], 
        nseq_other_abbrevs = []): 
        """ 
        Initialiser. It contains the following arguments.
        nseq_name: the full name of the note sequence.
        nseq_type: the type of note sequence (e.g., NSEQ_SCALE or NSEQ_CHORD).
        nseq_temp: the musical temperament from which the sequence of notes is
            taken. This should be an instance of the class temperament.
        nseq_posn: a sequence of integers representing positions of the notes
            in the sequence relative to its first note. (For best results,
            the first item should be 0 to represent the first note.) 
        nseq_nat_posns: a sequence of integers (which should be the same size
            as nseq_posn). For each item in nseq_posn, the corresponding item
            in nseq_nat_posns represent the difference in its calculated 
            natural note from the base key's natural note. For example, this
            parameter is [0, 2, 4] for a major chord; the second note is always
            two letters higher than the first note, and the third note is two
            letters higher again 
        nseq_abbrv: the primary abbreviation for the note sequence (if any).
        nseq_synonyms: a list of possible synonyms, or extra names, for the 
            note sequence. For example, the "Ionian" mode is another name for
            the major scale as far as this class is concerned.
        nseq_other_abbrevs: a list of possible alternate abbreviations for the
            note sequence. For example, both "min7" and "m7" are abbreviations
            for the minor seventh chord.

        Examples of defining scales:
        Major scale: noteseq("Major", NSEQ_SCALE, WestTemp, [0, 2, 4, 5, 7,
            9, 11], range(7), nseq_synonyms = ["Ionian"]);
        Harmonic Minor scale: noteseq("Harmonic Minor", NSEQ_SCALE, WestTemp,
            [0, 2, 3, 5, 7, 8, 11], range(7));

        Examples of defining chords:
        Major chord: noteseq("Major", NSEQ_CHORD, WestTemp, [0, 4, 7], 
            [0, 2, 4], "maj");
        Minor chord: noteseq("Minor", NSEQ_CHORD, WestTemp, 
            [0, 3, 7], [0, 2, 4], "min", nseq_other_abbrevs = ["m"]);
        Min 7 chord: noteseq("Minor Seventh", NSEQ_CHORD, WestTemp, 
            [0, 3, 7, 11], [0, 2, 4, 6], "min7", nseq_other_abbrevs = ["m7"]);

        Note: it is easier to define scales using the noteseq_scale class (this
        file) and chord using the noteseq_chord class (in chords.py)
        """
        self.nseq_name = nseq_name; 
        self.nseq_type = nseq_type;
        self.nseq_temp = nseq_temp;
        self.nseq_posn = nseq_posn;
        self.nseq_nat_posns = nseq_nat_posns;
        self.nseq_abbrev = nseq_abbrev;
        self.nseq_synonyms = nseq_synonyms;
        self.nseq_other_abbrevs = nseq_other_abbrevs;
        self.register_with_temp();

# We need to work out something here.        

    def register_with_temp(self):
        """ Registers this note sequence with the underlying temperament, so
            that it can be looked up by name, by abbreviation or by sequence. 
        """
        if self.nseq_synonyms:
            if self.nseq_name not in self.nseq_synonyms:
                our_names = [self.nseq_name] + self.nseq_synonyms;
            else:
                our_names = self.nseq_synonyms;
        else:
            our_names = [self.nseq_name];
        if self.nseq_other_abbrevs:
            if self.nseq_abbrev not in self.nseq_other_abbrevs:
                our_abbrevs = [self.nseq_abbrev] + self.nseq_other_abbrevs;
            else:
                our_abbrevs = self.nseq_other_abbrevs;
        else:
            our_abbrevs = [self.nseq_abbrev];
        self.nseq_temp.seq_maps.add_elem(self, self.nseq_type, our_names, 
            our_abbrevs, self.nseq_posn)

    def get_notes_for_key(self, key, rotate_by=0, slice=None):
        """ Get the notes for the note_seq starting from a given key. Then
            rotates the sequence (by rotate_by). Then slices it (using the
            argument in the slice parameter).
        """
        default_seq = self.nseq_temp.get_note_sequence(key, self.nseq_posn, 
            self.nseq_nat_posns); #C
        modulus = self.nseq_temp.no_nat_keys; 
        if slice:
            return multislice(default_seq, slice, offset=rotate_by, 
                mod=modulus);
        else:
            return rotate(default_seq, rotate_by);

    def get_posn_for_offset(self, rotate_by=0, slice=None, raz = False):
        """ For this noteseq, output its positions relative to its first
            note, then rotates the sequence (by rotate_by), then slices
            it (using slice, if not none). If and only if raz is True, the
            result is rotated-and-zeroed.
        """
        if slice:
            modulus = self.nseq_temp.no_nat_keys; 
            multisliced = multislice(self.nseq_posn, slice, offset=rotate_by,
                mod = modulus);
            if raz:
                return rotate_and_zero(multisliced, 0, self.nseq_temp.no_keys);
            else:
                return multisliced;
        else:
            if raz:
                return rotate_and_zero(self.nseq_posn, rotate_by, 
                    self.nseq_temp.no_keys);
            else:
                return rotate(self.nseq_posn, rotate_by);

class noteseq_scale(noteseq):
    """ A specialisation of noteseq used exclusively for defining scales -
        especially heptatonic/diotonic scales.
    """
    def __init__(self, nseq_name, nseq_temp, nseq_posn, nseq_nat_posns, 
        nseq_modes = []):
        """ Arguments:
            nseq_name: name of scale.
            nseq_temp: temperament for scale.
            nseq_posn: position of notes in scale.
            nseq_nat_posns: natural note positions for notes in scales.
            nseq_modes: a sequence of modes for the scale, listed in order of
                position.
        """
        if not nseq_modes:
            noteseq.__init__(self, nseq_name, NSEQ_SCALE, nseq_temp, 
                nseq_posn, nseq_nat_posns);
            return;
        noteseq.__init__(self, nseq_name, NSEQ_SCALE, nseq_temp, nseq_posn, 
            nseq_nat_posns, "", [nseq_modes[0]]); 
        new_nseq_pos = nseq_posn;
        new_nseq_nat_pos = nseq_nat_posns;
        
# This creates more instances of noteseq_scales - all for the different modes.
# By creating them, they will be automatically connected to the appropriate
# temperament's dictionary.
        
        for i in range(1, len(nseq_posn)):
            new_nseq_pos = rotate_and_zero(new_nseq_pos, 1, 
                nseq_temp.no_keys);
            new_nseq_nat_pos = rotate_and_zero(new_nseq_nat_pos, 1,
                nseq_temp.no_nat_keys);
            OurNewScale = noteseq_scale(nseq_modes[i], nseq_temp, new_nseq_pos,
                new_nseq_nat_pos);

    def __unicode__(self):
        return unicode(nseq_name);

# The following scales and names are derived from:
# http://docs.solfege.org/3.9/C/scales/modes.html

# The only exception is the "Discordant Minor", which I invented for shit and
# giggles.

# Note: MelMinScale represents a melodic minor scale _ascending_. See the
# Aeolian mode of the Major scale for melodic minor scale _descending_.

HEPT_NAT_POSNS = range(7);
MEL_MIN_NOTE_POS = [0, 2, 3, 5, 7, 9, 11];
HARM_MIN_NOTE_POS = [0, 2, 3, 5, 7, 8, 11];
HARM_MAJ_NOTE_POS = [0, 2, 4, 5, 7, 8, 11];
DISC_MIN_NOTE_POS = [0, 2, 3, 5, 6, 9, 11];
HUNGARIAN_NOTE_POS = [0, 3, 4, 6, 7, 9, 10];

# For ease of comprehension, we have the list of modes as arrays which can be 
# browsed from outside.

MAJORMODES = ["Ionian",
    "Dorian", 
    "Phrygian", 
    "Lydian", 
    "Mixolydian", 
    "Aeolian", 
    "Locrian"];
    
MELMINORMODES = ["Jazz Minor",
    "Dorian " + M_FLAT + "9", 
    "Lydian Augmented", 
    "Lydian Dominant", 
    "Mixolydian " + M_FLAT + "13", 
    "Semilocrian", 
    "Superlocrian"];

HARMINORMODES = ["Harmonic Minor", 
    "Locrian " + M_SHARP + "6", 
    "Ionian Augmented", 
    "Romanian", 
    "Phrygian Dominant", 
    "Lydian " + M_SHARP + "2", 
    "Ultralocrian"];
    
HARMMAJORMODES = ["Harmonic Major", 
    "Dorian " + M_FLAT + "6", 
    "Phrygian " + M_FLAT + "4", 
    "Lydian " + M_FLAT + "3", 
    "Mixolydian " + M_FLAT + "9", 
    "Lydian " + M_SHARP + "2 " + M_SHARP + "5",
    "Locrian " + M_FLAT + M_FLAT + "7"];

DISCORDMINMODES = ["Melodic Minor " + M_FLAT + "5", 
    "Dorian " + M_FLAT + "9 " + M_FLAT + "4", 
    "Minor Lydian Augmented", 
    "Lydian Dominant " + M_FLAT + "9", 
    "Lydian Augmented " + M_SHARP + "2 " + M_SHARP + "3", 
    "Semilocrian " + M_FLAT + M_FLAT + "7",
    "Superlocrian " + M_FLAT + M_FLAT + "6"];
    
HUNGARIANMODES = ["Hungarian", 
    "Superlocrian " + M_FLAT + M_FLAT + "6 " + M_FLAT + M_FLAT + "7", 
    "Harmonic Minor " + M_FLAT + "5", 
    "Superlocrian " + M_SHARP + "6", 
    "Melodic Minor " + M_SHARP + "5", 
    "Dorian " + M_FLAT + "9 " + M_SHARP + "11",
    "Lydian Augmented " + M_SHARP + "3"];    

MajorScale = noteseq_scale("Major", WestTemp, CHROM_NAT_NOTE_POS, 
    HEPT_NAT_POSNS, MAJORMODES);

MelMinorScale = noteseq_scale("Melodic Minor", WestTemp, MEL_MIN_NOTE_POS, 
    HEPT_NAT_POSNS, MELMINORMODES);

HarmMinorScale = noteseq_scale("Harmonic Minor", WestTemp, HARM_MIN_NOTE_POS,
    HEPT_NAT_POSNS, HARMINORMODES); 

HarmMajorScale = noteseq_scale("Harmonic Major", WestTemp, HARM_MAJ_NOTE_POS,
    HEPT_NAT_POSNS, HARMMAJORMODES); 

DiscMinorScale = noteseq_scale("Discordant Minor", WestTemp, DISC_MIN_NOTE_POS,
    HEPT_NAT_POSNS, DISCORDMINMODES); 
    
HungarianScale = noteseq_scale("Hungarian", WestTemp, HUNGARIAN_NOTE_POS,
    HEPT_NAT_POSNS, HUNGARIANMODES);     

def scale_analysis(scale_name, perm_chord_type):
    """ This is for decomposing a scale into chords. We use this as a test
    routine so we know what is required.
    """
    # 1. Lookup scale name.
    noteseq_founditem = noteseq.find_by_name(scale_name);
    # 2. Find scale pattern.
    ournoteseq = noteseq_founditem.noteseq;
    noteseq_founditem.noteseq.getpattern(noteseq_founditem.indices);
    for i in range(ournoteseq.nseq_posn.len()):
        ournoteslice = noteseq_for_slice(i, perm_chord_type);
        notechord_founditem = noteseq.find_by_chord(ournoteslice);
        if notechord_founditem == None:
            print "None";
        else:
            print notechord_founditem.getName();
            
    return;
    
if __name__ == "__main__":
    class TestNoteSeqAndScales(unittest.TestCase):
        """ This tests the note_seq and scales class. """
        
        def setUp(self):
            self.majorscale = MajorScale;
            self.melminscale = MelMinorScale;
            self.harmminscale = HarmMinorScale;
            self.harmmajscale = HarmMajorScale;
            self.dorianscale = WestTemp.get_nseqby_seqpos(
                [0, 2, 3, 5, 7, 9, 10], NSEQ_SCALE);
            self.locrianscale = WestTemp.get_nseqby_seqpos(
                [0, 1, 3, 5, 6, 8, 10], NSEQ_SCALE);
            self.superlocrianscale = WestTemp.get_nseqby_seqpos(
                [0, 1, 3, 4, 6, 8, 10], NSEQ_SCALE);
            self.ultralocrianscale = WestTemp.get_nseqby_seqpos(
                [0, 1, 3, 4, 6, 8, 9], NSEQ_SCALE);
            self.dubflat7locrianscale = WestTemp.get_nseqby_seqpos(
                [0, 1, 3, 5, 6, 8, 9], NSEQ_SCALE);
            self.majorchord = noteseq("TestMajor", NSEQ_SCALE, WestTemp, 
                [0, 4, 7], [0, 2, 4]);
            
        def test_temperament_dict(self):
            """ Like test_seq_dict, but checks dictionary in WestTemp. """
            self.assertTrue(WestTemp.check_nseqby_subdict(NSEQ_CHORD));
            self.assertTrue(WestTemp.check_nseqby_subdict(NSEQ_SCALE));
            self.assertFalse(WestTemp.check_nseqby_subdict(666));
            self.assertTrue(WestTemp.check_nseqby_name(NSEQ_SCALE, "Major"));
            self.assertFalse(WestTemp.check_nseqby_name(NSEQ_CHORD, "Minor"));
            self.assertFalse(WestTemp.check_nseqby_name(NSEQ_SCALE, "Minor"));
            self.assertFalse(WestTemp.check_nseqby_abbrv(NSEQ_SCALE, 
                "maj"));
            self.assertFalse(WestTemp.check_nseqby_abbrv(NSEQ_CHORD, 
                "min"));
            self.assertFalse(WestTemp.check_nseqby_abbrv(NSEQ_SCALE, 
                "min"));
            self.assertFalse(WestTemp.check_nseqby_seqpos(NSEQ_CHORD, 
                CHROM_NAT_NOTE_POS));
            self.assertTrue(WestTemp.check_nseqby_seqpos(NSEQ_SCALE, 
                CHROM_NAT_NOTE_POS));
            self.assertEqual(WestTemp.get_nseqby_name("Major", NSEQ_SCALE), 
                self.majorscale);
            self.assertEqual(WestTemp.get_nseqby_name("Ionian", NSEQ_SCALE), 
                self.majorscale);
            self.assertEqual(WestTemp.get_nseqby_name("Melodic Minor", 
                NSEQ_SCALE), self.melminscale);
            self.assertEqual(WestTemp.get_nseqby_name("Jazz Minor", 
                NSEQ_SCALE), self.melminscale);
            self.assertEqual(WestTemp.get_nseqby_name("Harmonic Minor", 
                NSEQ_SCALE), self.harmminscale);
            self.assertEqual(WestTemp.get_nseqby_name("Harmonic Major", 
                NSEQ_SCALE), self.harmmajscale); 
            self.assertEqual(WestTemp.get_nseqby_name("Dorian", 
                NSEQ_SCALE), self.dorianscale);
            self.assertEqual(WestTemp.get_nseqby_name("Locrian", 
                NSEQ_SCALE), self.locrianscale);  
            self.assertEqual(WestTemp.get_nseqby_name("Superlocrian", 
                NSEQ_SCALE), self.superlocrianscale);  
            self.assertEqual(WestTemp.get_nseqby_name("Ultralocrian", 
                NSEQ_SCALE), self.ultralocrianscale);                  
            self.assertEqual(WestTemp.get_nseqby_name("Locrian " + M_FLAT \
                + M_FLAT + "7", NSEQ_SCALE), self.dubflat7locrianscale);

            self.assertEqual(WestTemp.get_nseqby_seqpos(
                CHROM_NAT_NOTE_POS, NSEQ_SCALE), self.majorscale);

            self.assertEqual(WestTemp.get_nseqby_seqpos(
                MEL_MIN_NOTE_POS, NSEQ_SCALE), self.melminscale);
            self.assertEqual(WestTemp.get_nseqby_seqpos(
                HARM_MIN_NOTE_POS, NSEQ_SCALE), self.harmminscale);
            self.assertEqual(WestTemp.get_nseqby_seqpos(
                HARM_MAJ_NOTE_POS, NSEQ_SCALE), self.harmmajscale);                

        def test_get_notes_for_key(self):
            self.assertEqual(self.majorchord.get_notes_for_key("Eb", 
                0, [0]), [u'E\u266d']);            
            self.assertEqual(self.majorchord.get_notes_for_key("Eb", 
                1, [0]), ['G']);            
            self.assertEqual(self.majorchord.get_notes_for_key("Eb", 
                2, [0]), [u'B\u266d']);            
            self.assertEqual(self.majorchord.get_notes_for_key("Eb", 
                3, [0]), [u'E\u266d']);            
            self.assertEqual(self.majorchord.get_notes_for_key("Eb", 
                0, [0, 2]), [u'E\u266d', u'B\u266d']);            
            self.assertEqual(self.majorchord.get_notes_for_key("Eb", 
                1, [0, 2]), ['G', u'E\u266d']);            
            self.assertEqual(self.majorchord.get_notes_for_key("Eb", 
                2, [0, 2]), [u'B\u266d', 'G']);            
            self.assertEqual(self.majorchord.get_notes_for_key("Eb", 
                3, [0, 2]), [u'E\u266d', u'B\u266d']);
            self.assertEqual(self.majorchord.get_notes_for_key("Eb", 
                0, [0, 1]), [u'E\u266d', 'G']);            
            self.assertEqual(self.majorchord.get_notes_for_key("Eb", 
                1, [0, 1]), ['G', u'B\u266d']);            
            self.assertEqual(self.majorchord.get_notes_for_key("Eb", 
                2, [0, 1]), [u'B\u266d', u'E\u266d',]);            
            self.assertEqual(self.majorchord.get_notes_for_key("Eb", 
                3, [0, 1]), [u'E\u266d', 'G']);                
            self.assertEqual(self.majorscale.get_notes_for_key("C", 
                3, [0, 1, 2, 3]), ['F', 'G', 'A', 'B']);  

        def test_get_posn_for_offset(self):
            self.assertEqual(self.majorchord.get_posn_for_offset(0, 
                raz=False), [0, 4, 7]);
            self.assertEqual(self.majorchord.get_posn_for_offset(0, 
                raz=True), [0, 4, 7]);            
            self.assertEqual(self.majorchord.get_posn_for_offset(0, [0], 
                raz=False), [0]);
            self.assertEqual(self.majorchord.get_posn_for_offset(0, [0], 
                raz=True), [0]);
            self.assertEqual(self.majorchord.get_posn_for_offset(0, [1], 
                raz=False), [4]);
            self.assertEqual(self.majorchord.get_posn_for_offset(0, [1], 
                raz=True), [0]);
            self.assertEqual(self.majorchord.get_posn_for_offset(0, [2], 
                raz=False), [7]);
            self.assertEqual(self.majorchord.get_posn_for_offset(0, [2], 
                raz=True), [0]);                
            self.assertEqual(self.majorchord.get_posn_for_offset(1, 
                raz=False), [4, 7, 0]);
            self.assertEqual(self.majorchord.get_posn_for_offset(1, 
                raz=True), [0, 3, 8]);            
            self.assertEqual(self.majorchord.get_posn_for_offset(1, [0], 
                raz=False), [4]);
            self.assertEqual(self.majorchord.get_posn_for_offset(1, [0], 
                raz=True), [0]);
            self.assertEqual(self.majorscale.get_posn_for_offset(0, 
                [0, 2, 4], raz=True), [0, 4, 7]);
            self.assertEqual(self.majorscale.get_posn_for_offset(1, 
                [0, 2, 4], raz=True), [0, 3, 7]);                 
            self.assertEqual(self.majorscale.get_posn_for_offset(3, 
                [0, 2, 4], raz=True), [0, 4, 7]);                
            self.assertEqual(self.majorscale.get_posn_for_offset(4, 
                [0, 2, 4], raz=True), [0, 4, 7]);
            self.assertEqual(self.majorscale.get_posn_for_offset(6, 
                [0, 2, 4], raz=True), [0, 3, 6]);                

    unittest.main()

    
    
    



#!/usr/bin/env python
     
from distutils.core import setup

setup(
    name='musictheory',
    version='0.4',
    author='Peter Murphy',
    author_email='peterkmurphy@gmail.com',
    packages=['musictheory'],
    url='http://pypi.python.org/pypi/musictheory/',
    license='LICENSE.txt',
    description='Provides classes for "music set theory"-like analysis.',
    keywords='music set theory temperament scale chords',
    long_description=open('README.txt').read(),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Other Audience',
        'License :: OSI Approved :: BSD License',
        'Operating System :: MacOS',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Topic :: Artistic Software',
        'Topic :: Education',
        'Topic :: Multimedia :: Sound/Audio',
        'Topic :: Other/Nonlisted Topic',
        'Topic :: Software Development',
        'Topic :: Utilities',
        ],
)     



#!/usr/bin/python 
#-*- coding: UTF-8 -*-
# temperament.py: Representing musical temperaments (and storing sequences
# associated with them).
#
# Copyright (c) 2008-2013 Peter Murphy <peterkmurphy@gmail.com>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * The names of its contributors may not be used to endorse or promote 
#       products derived from this software without specific prior written 
#       permission.
#
# THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ''AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# Too many ideas, so let's work on the fundamentals.
# A musical temperament consists of the following things - keys, and their 
# positions within it. For example, we will start with the 12 chromatic
# scale. We only need to specify the seven natural keys, and their position
# within it. Doing anymore is overkill. For example, this code does not concern
# itself with the frequencies of notes in Hz.

import re;
import unittest;
from musutility import printunicode, rotate, multislice, repseq;

# Unicode string constants for music notation.

M_SHARP = u"\u266f";
M_NATURAL = u"\u266e";
M_FLAT = u"\u266d";

# Since this may not display on browsers, there are non-Unicode equivalents.

MNU_SHARP = "#";
MNU_FLAT = "b";
MNU_NATURAL = "n";

# This constant is used for parsing note names into keys.

RE_NOTEPARSE = u"(?P<basenotename>[A-Z](#|b|n|" + M_FLAT + "|" + M_NATURAL \
    + "|"+ M_SHARP + ")*)";

# Constants used for the nseq_type arguments in noteseqs. See scales.py for
# more information.

NSEQ_SCALE = 0; # Used for specifying scales.
NSEQ_CHORD = 1; # Used for specifying chords.

# Several competing approaches were timed for character replacement in Python
# strings.  The quickest result came from using the replace function. For more
# information, see: 
# http://stackoverflow.com/questions/2484156/ \
#    is-str-replace-replace-ad-nauseam-a-standard-idiom-in-python

def un_unicode_accdtls(str_note):
    """ Turns all Unicode musical characters in str_note to their ASCII 
        equivalent, and returns the result.
    
        Note: the function is intended for viewing musical scales through
        deficient browsers like IE 6 and 7. Characters may appear as blocks
        in these environments.
    """
    return str_note.replace(M_FLAT, MNU_FLAT).replace(M_SHARP, 
        MNU_SHARP).replace(M_NATURAL, MNU_NATURAL);

# The following class seq_dict is used as a multiple key map to noteseq 
# instances, which are defined in scales.poy. However, seq_dict instances are
# stored with temperament objects, which is why they are stored here.

class seq_dict():
    """ The seq_dict class acts as a dictionary for noteseq instances. It
        allows users and programmers to look up noteseqs by name, by
        abbreviation, and by the sequence of integers representing the
        positions in the sequence. Each seq_dict should be assigned to one
        dictionary.
    """
    
    class nseqtype_ins_dict():
        """ The nseqtype_ins_dict class represents a subdictionary of noteseq
            instances sharing the same nseq_type. It is useful to split up
            noteseqs by nseq_type, lest property lookup return the "wrong"
            object. For example, a major 13 chord has the same pattern of 
            integral positions as the major scale; i.e., [0, 2, 4, 5, 7, 9, 
            11]. To look up the correct object of the two by pattern, the
            nseq_type must be specified.
        """
        def __init__(self, nseq_type):
            """ The constructed for nseqtype_ins_dict. This sets up seperate
                dictionaries for names, abbreviations and integral position
                dictionaries.
            """
            self.nseq_type = nseq_type;
            self.name_dict = {};
            self.abbrv_dict = {};
            self.seqpos_dict = {};
            
    def __init__(self, nseq_types, nseq_temp):
        """ The constructor for seq_dict. Arguments:
            nseq_types: a list of possible nseq_type values for lookups.
            nseq_temp: the associated temperament object.
        """
        self.nseq_temp = nseq_temp;
        self.nseqtype_maps = {};
        for i in nseq_types:
            self.nseqtype_maps[i] = seq_dict.nseqtype_ins_dict(i);

    def add_elem(self, elem, nseq_type, name_s, abbrv_s, seqpos):
        """ This adds an element to seq_dict. The arguments:
            elem: the element to add to seq_dict instance.
            nseq_type: the type of the elemenet (such as scale or chord).
            name_s: a string, or a sequence of strings. This provides names
                as keys that map onto elem.
            abbrv_s: a string, or a sequence of strings. This provides 
                abbreviations as keys that map onto elem.
            seqpos: a sequence. A tuple form will be used as a key that
                maps onto elem.
                
            If nseq_type is not associated with any of the sub-dictionaries in
            seq_dict, then this function exits.
        """
        if not self.nseqtype_maps.has_key(nseq_type):
            return False;
        sub_dictionary = self.nseqtype_maps[nseq_type];
        
# PKM - suspicious code. Replace:        
        
#        sub_dictionary.seqpos_dict[tuple(seqpos)] = elem;

# By:
        ourmod = self.nseq_temp.no_keys;
        ourtuple = tuple(sorted([i % ourmod for i in seqpos]));
        sub_dictionary.seqpos_dict[ourtuple] = elem;


        if isinstance(name_s, basestring):
            sub_dictionary.name_dict[name_s] = elem;
        else:
            for s in name_s:
                sub_dictionary.name_dict[s] = elem;
        if isinstance(abbrv_s, basestring):
            sub_dictionary.abbrv_dict[abbrv_s] = elem;
        else:
            for s in abbrv_s:
                sub_dictionary.abbrv_dict[s] = elem;
        return True;

# The next few functions, while relatively trivial, at least save some typing
# time in practice.

    def check_nseqby_subdict(self, nseq_type):
        """ Checks if there is a subdictionary associated with nseq_type. """
        return self.nseqtype_maps.has_key(nseq_type);

    def check_nseqby_name(self, nseq_type, name):
        """ Checks if there is a noteseq with a given name. """
        return self.nseqtype_maps[nseq_type].name_dict.has_key(name);
        
    def check_nseqby_abbrv(self, nseq_type, abbrv):
        """ Checks if there is a noteseq with a given abbreviation. """
        return self.nseqtype_maps[nseq_type].abbrv_dict.has_key(abbrv);

    def check_nseqby_seqpos(self, nseq_type, seqpos):
        """ Checks if there is a noteseq with a given sequence position. """
        sortedseqpos = tuple(sorted(seqpos));
        return self.nseqtype_maps[nseq_type].seqpos_dict.has_key(sortedseqpos);    

    def get_nseqby_name(self, name, nseq_type):
        """ Looks up a noteseq by name. """
        return self.nseqtype_maps[nseq_type].name_dict[name];
        
    def get_nseqby_abbrv(self, abbrv, nseq_type):
        """ Looks up a noteseq by abbreviation. """
        return self.nseqtype_maps[nseq_type].abbrv_dict[abbrv];

    def get_nseqby_seqpos(self, seqpos, nseq_type):
        """ Looks up a noteseq by sequence position. """
        sortedseqpos = tuple(sorted(seqpos));
        return self.nseqtype_maps[nseq_type].seqpos_dict[sortedseqpos];

class temperament():
    """ A musical temperament is used to define the possible keys in music, 
        and also the positions of keys relevant to each other. Our primary
        example is the western or chromatic temperament, with 12 possible keys.
        A temperament also defines which of the keys can be represented as
        naturals (like "C", "D" and "E"), and which need to be represented with
        accidentals (like "C#" and "Db".
        
        Doing anything more at the moment is overkill. For example, we are not
        intereted in the possible frequencies for notes of a given key.
    """
    
    def __init__(self, no_keys, nat_keys, nat_key_posn):
        """ Initialiser. It contains the following arguments.
            no_keys: the number of keys in the temperament. 
            nat_keys: an array consisting of the names of the natural 
                (unsharped or unflattened) keys in the temperament.
            nat_key_posn: the position of the natural keys in the temperament.
                These should correspond to the elements in nat_keys.
                Positions are calculated base zero. 
        """
        self.no_keys = no_keys;
        self.nat_keys = nat_keys;
        self.nat_key_posn = nat_key_posn;
        
# Extra variable: no_nat_keys: number of natural keys.        
        
        self.no_nat_keys = min(len(nat_keys), len(nat_key_posn));
        
# Extra variable: nat_key_pos_lookup: key -> position (e.g. A -> 9, C->0).        
        
        self.nat_key_pos_lookup = {};
        
# Extra variable: post_lookup_nat_key: position -> key/None (e.g., 9->A, 0->C).        
        
        self.pos_lookup_nat_key = {};
        
# Extra variable: nat_key_lookup_order: looks up order of nat_keys (e.g, C->0,
# D->1... B->6). Reverse translation available by self.nat_keys[order].

        self.nat_key_lookup_order = {};
        
        for i in range(self.no_nat_keys):
            self.nat_key_pos_lookup[nat_keys[i]] = nat_key_posn[i];
            self.pos_lookup_nat_key[nat_key_posn[i]] = nat_keys[i];
            self.nat_key_lookup_order[nat_keys[i]] = i;
        self.parsenote = re.compile(RE_NOTEPARSE);
        
#  Useful for dictionary lookup later.       
        
        self.seq_maps = seq_dict([NSEQ_SCALE, NSEQ_CHORD], self);

    def note_parse(self, key):
        """ Parses the name of a key into the tuple (natural key name,
            sharp_or_flat count). For example "C#" is parsed into ("C", 1),
            "Db" is parsed into ("D", -1), and "E" is parsed into ("E", 0).
            As the reader may gather, negative numbers are used for 
            flattened notes.
        """
        noteMatch = self.parsenote.match(key);
        if noteMatch == None:
            return None;
        noteGroup = noteMatch.group('basenotename');
        baseNote = noteGroup[0];
        flatSharpAdj = 0;
        for i in noteGroup[1:]:
            if i in (M_FLAT + MNU_FLAT):
                flatSharpAdj = flatSharpAdj - 1;
            if i in (M_SHARP + MNU_SHARP):
                flatSharpAdj = flatSharpAdj + 1;
        return (baseNote, flatSharpAdj,);            

    def get_pos_of_key(self, key):
        """ Returns the position of the key in the temperament. """
        key_parsed = self.note_parse(key); 
        return self.nat_key_pos_lookup[key_parsed[0]] + key_parsed[1];

    def get_key_of_pos(self, pos, desired_nat_note = None, 
            sharp_not_flat = True):
        """ Given a position in the temperament, this function attempts to
            return the "best" key name corresponding to it. This is not a 
            straight-forward reverse of get_pos_of_key, as there may be two
            different keynames for the same position, with one preferred. For
            example, "C# and "Db" are the same key, but "C# is preferred in an
            A major scale. Fortunately, arguments are provided to indicate the
            programmer's preference.
            pos: the position inside the temperament.
            desired_nat_note: the preferred natural key to start the key name.
                For example "C" makes the function return "C#" instead of "Db".
                If None, then the preference depends on...
            sharp_not_flat: if True, returns the sharpened accidental form 
                (e.g., "C#"); if False, returns the flattened accidental form
                (i.e., "Db").
        """

# accdtls: number of sharps or flats to add to the output string.

        if desired_nat_note:
            accdtls = pos - self.nat_key_pos_lookup[desired_nat_note];
            accdtls = (accdtls % self.no_keys);
            if accdtls > (self.no_keys / 2):
                accdtls = accdtls - self.no_keys;
            if accdtls > 0:
                return desired_nat_note + (M_SHARP * accdtls);
            elif accdtls < 0:
                return desired_nat_note + (M_FLAT * (-1 * accdtls));
            else:
                return desired_nat_note;
        else:
            accdtls = 0;
            if self.pos_lookup_nat_key.has_key(pos % self.no_keys):
                return self.pos_lookup_nat_key[pos % self.no_keys];
            elif sharp_not_flat:
                while accdtls < self.no_keys:
                    accdtls = accdtls - 1;
                    if self.pos_lookup_nat_key.has_key((pos + accdtls) 
                        % self.no_keys):
                        return self.pos_lookup_nat_key[(pos + accdtls) 
                            % self.no_keys] + (M_SHARP * (-1 * accdtls));
            else:
                while accdtls < self.no_keys:
                    accdtls = accdtls + 1;
                    if self.pos_lookup_nat_key.has_key((pos + accdtls) 
                        % self.no_keys):
                        return self.pos_lookup_nat_key[(pos + accdtls) 
                            % self.no_keys] + (M_FLAT * accdtls);
        return None;

    def get_note_sequence(self, key, pos_seq, nat_pos_seq = None, 
        sharp_not_flat = True):
        """ This function takes a key and a sequence of positions relative to
            it. It returns a sequence of notes. Arguments:
            key: the starting key; examples are "C", "C#" and "Db".
            pos_seq: a list of positions relative to it in the temperament 
                base 0). For example, in the Western/Chromatic temperament, a
                key of "C" and a sequence of [0, 1] returns ["C", "C#].
            nat_pos_seq: a list of numbers. These are used to calculate the
                desired natural notes produced from the corresponding positions
                in pos_seq. A number of 0 means to use the same natural note as
                in key, a number of 1 means using the next natural note, and so
                on. Examples for the Western/Chromatic scale:
                    get_note_sequence("C", [0, 1], [0, 0]) -> ["C", "C#"]
                    get_note_sequence("C", [0, 1], [0, 1]) -> ["C", "Dbb"]
                    get_note_sequence("C", [0, 1], [0, 6]) -> ["C", "B##"]
            sharp_not_flat: if nat_pos_seq is None, indicates whether notes 
                with accidentals are preferred to have sharps (True) or flats
                (False)
        """
        base_key_pos = self.get_pos_of_key(key); 
        result_pos_seq = [((base_key_pos + i) % self.no_keys) for i in pos_seq];
        if nat_pos_seq:
            result_seq = [];
            desired_nat_key = self.note_parse(key)[0];
            desired_nat_key_posn = self.nat_keys.index(desired_nat_key);
            for i in range(min(len(nat_pos_seq), len(result_pos_seq))):
                des_nat_note = self.nat_keys[(desired_nat_key_posn
                    + nat_pos_seq[i]) % self.no_nat_keys];
                des_key = self.get_key_of_pos(result_pos_seq[i], des_nat_note);
                result_seq.append(des_key);
            return(result_seq);
        else:
            return [self.get_key_of_pos(i, None, sharp_not_flat) for i in
                result_pos_seq];

    def get_keyseq_notes(self, note_seq):
        """ The reverse of get_note_sequence. This takes a sequence of notes
            (note_seq) and converts it into [base key, position sequence];
            this is returned by the function.
        """
        base_key = note_seq[0];
        base_pos = self.get_pos_of_key(base_key);
        pos_seq = [((self.get_pos_of_key(i) - base_pos) % self.no_keys) for
            i in note_seq];
        return [base_key, pos_seq];

# The next few functions are rip-offs of seq_dict functions.

    def add_elem(self, elem, nseq_type, name_s, abbrv_s, seqpos):
        """ This adds an element to the dictionary inside the temperament. The
            arguments:
            elem: the element to add  to the dictionary .
            nseq_type: the type of the elemenet (such as scale or chord).
            name_s: a string, or a sequence of strings. This provides names
                as keys that map onto elem.
            abbrv_s: a string, or a sequence of strings. This provides 
                abbreviations as keys that map onto elem.
            seqpos: a sequence. A tuple form will be used as a key that
                maps onto elem.
                
            If nseq_type is not associated with any of the sub-dictionaries in
            the dictionary in the temperament, then this function exits.
        """
        return self.seq_maps.add_elem(elem, nseq_type, name_s, abbrv_s, 
            seqpos);

    def check_nseqby_subdict(self, nseq_type):
        """ Checks if there is a subdictionary associated with nseq_type. """
        return self.seq_maps.check_nseqby_subdict(nseq_type);

    def check_nseqby_name(self, nseq_type, name):
        """ Checks if there is a noteseq with a given name. """
        return self.seq_maps.check_nseqby_name(nseq_type, name);
        
    def check_nseqby_abbrv(self, nseq_type, abbrv):
        """ Checks if there is a noteseq with a given abbreviation. """
        return self.seq_maps.check_nseqby_abbrv(nseq_type, abbrv);

    def check_nseqby_seqpos(self, nseq_type, seqpos):
        """ Checks if there is a noteseq with a given sequence position. """
        return self.seq_maps.check_nseqby_seqpos(nseq_type, seqpos); 

    def get_nseqby_name(self, name, nseq_type):
        """ Looks up a noteseq (or anything else) by name. """
        if self.seq_maps.check_nseqby_name(nseq_type, name):
            return self.seq_maps.get_nseqby_name(name, nseq_type);
        else:
            return None;
        
    def get_nseqby_abbrv(self, abbrv, nseq_type):
        """ Looks up a noteseq (or anything else) by abbreviation. """
        if self.seq_maps.check_nseqby_abbrv(nseq_type, abbrv):
            return self.seq_maps.get_nseqby_abbrv(abbrv, nseq_type);
        else:
            return None;

    def get_nseqby_seqpos(self, seqpos, nseq_type):
        """ Looks up a noteseq (or anything else) by sequence position. """
        if self.seq_maps.check_nseqby_seqpos(nseq_type, seqpos):
            return self.seq_maps.get_nseqby_seqpos(seqpos, nseq_type);
        else:
            return None;


# We immediately use this class to define a Western or Chromatic temperament
# object. Apart from useful for understanding the class, the object is used in
# testing.

CHROM_NAT_NOTES = ["C", "D", "E", "F", "G", "A", "B"];
CHROM_NAT_NOTE_POS = [0, 2, 4, 5, 7, 9, 11];
CHROM_SIZE = 12;
WestTemp = temperament(CHROM_SIZE, CHROM_NAT_NOTES, CHROM_NAT_NOTE_POS);

if __name__ == "__main__":

    class TestTemperament(unittest.TestCase):
        """ This tests the temperament class. it also tests the seq_dict
            class and un_unicode_accdtls function, as they are associated. 
        """
        
        def setUp(self):
            self.chromabsrep = ["C", "C" + M_SHARP, "D", "E" + M_FLAT, "E", 
                "F", "F" + M_SHARP, "G", "A" + M_FLAT, "A", "B" + M_FLAT, 
                "B"+M_NATURAL];
            self.chrom_as_sharp = ["C", "C" + M_SHARP, "D", "D" + M_SHARP, "E", 
                "F", "F" + M_SHARP, "G", "G" + M_SHARP, "A", "A" + M_SHARP, 
                "B"]; 
            self.chrom_as_flat = ["C", "D" + M_FLAT, "D", "E" + M_FLAT, "E", 
                "F", "G" + M_FLAT, "G", "A" + M_FLAT, "A", "B" + M_FLAT, 
                "B"];
            self.triad_pattern = [0, 2, 4];
            self.major_interval = [0, 4, 7]; 
            
            self.ourseqdict = seq_dict([NSEQ_SCALE, NSEQ_CHORD], WestTemp);
            self.majorchord = ["Major", "maj", self.major_interval];
            self.majorscale = ["Major", [], CHROM_NAT_NOTE_POS];
            self.seq_maps = seq_dict([NSEQ_SCALE, NSEQ_CHORD], WestTemp);
            self.seq_maps.add_elem(self.majorchord, NSEQ_CHORD, "Major", "maj", 
                self.major_interval);
            self.seq_maps.add_elem(self.majorscale, NSEQ_SCALE, "Major", [], 
                CHROM_NAT_NOTE_POS);                
                
        def test_temperament_init_(self):
            self.assertEqual(WestTemp.no_keys, CHROM_SIZE);
            self.assertEqual(WestTemp.no_nat_keys, 7);
            self.assertEqual(WestTemp.nat_keys, CHROM_NAT_NOTES);
            self.assertEqual(WestTemp.nat_key_posn, CHROM_NAT_NOTE_POS);
            for i in range(7):
                note_desired = CHROM_NAT_NOTES[i];
                note_index = WestTemp.nat_key_pos_lookup[note_desired];
                self.assertEqual(WestTemp.pos_lookup_nat_key[note_index],
                    note_desired);
            
        def test_temperament_get_pos_of_key(self):
            for i in range(CHROM_SIZE):
                self.assertEqual(WestTemp.get_pos_of_key(self.chromabsrep[i]),
                    i);

        def test_temperament_get_key_of_pos(self):
            for i in range(CHROM_SIZE):
                self.assertEqual(WestTemp.get_key_of_pos(i, None, True),
                    self.chrom_as_sharp[i]);            
                self.assertEqual(WestTemp.get_key_of_pos(i, None, False),
                    self.chrom_as_flat[i]);
                if i < 6:
                    self.assertEqual(WestTemp.get_key_of_pos(i, "C", False),
                        "C" + (M_SHARP * i));
                    self.assertEqual(WestTemp.get_key_of_pos(i, "B", False),
                        "B" + (M_SHARP * (i + 1)));
                elif i == 6:
                    self.assertEqual(WestTemp.get_key_of_pos(i, "C", False),
                        "C" + (M_SHARP * 6));
                    self.assertEqual(WestTemp.get_key_of_pos(i, "B", False),
                        "B" + (M_FLAT * 5));
                else:
                    self.assertEqual(WestTemp.get_key_of_pos(i, "C", False),
                        "C" + (M_FLAT * (12 - i)));
                    self.assertEqual(WestTemp.get_key_of_pos(i, "B", False),
                        "B" + (M_FLAT * (11 - i)));

        def test_get_note_sequence(self):
            self.assertEqual(WestTemp.get_note_sequence("C", 
                self.major_interval, None, False), ["C", "E", "G"]);
            self.assertEqual(WestTemp.get_note_sequence("C", 
                self.major_interval, None, True), 
                ["C", "E", "G"]);                
            self.assertEqual(WestTemp.get_note_sequence("C", 
                self.major_interval, self.triad_pattern, True), 
                ["C", "E", "G"]);              
            self.assertEqual(WestTemp.get_note_sequence("C#", 
                self.major_interval, None, False), 
                ["D" + M_FLAT, "F", "A" + M_FLAT]);
            self.assertEqual(WestTemp.get_note_sequence("C#", 
                self.major_interval, None, True), 
                ["C" + M_SHARP, "F", "G" + M_SHARP]);                
            self.assertEqual(WestTemp.get_note_sequence("C#", 
                self.major_interval, self.triad_pattern, True), 
                ["C" + M_SHARP, "E" + M_SHARP, "G" + M_SHARP]); 

            self.assertEqual(WestTemp.get_note_sequence("Db", 
                self.major_interval, None, False), 
                ["D" + M_FLAT, "F", "A" + M_FLAT]);
            self.assertEqual(WestTemp.get_note_sequence("Db", 
                self.major_interval, None, True), 
                ["C" + M_SHARP, "F", "G" + M_SHARP]);                
            self.assertEqual(WestTemp.get_note_sequence("Db", 
                self.major_interval, self.triad_pattern, True), 
                ["D" + M_FLAT, "F", "A" + M_FLAT]); 
         
        def test_get_keyseq_notes(self):            
            self.assertEqual(WestTemp.get_keyseq_notes(["C", "E", "G"]), 
                ["C", [0, 4, 7]]);
            self.assertEqual(WestTemp.get_keyseq_notes(["C" + M_SHARP,
                "F", "A" + M_FLAT]), ["C" + M_SHARP, [0, 4, 7]]);                  
            self.assertEqual(WestTemp.get_keyseq_notes(["D" + M_FLAT,
                "F", "A" + M_FLAT]), ["D" + M_FLAT, [0, 4, 7]]);  

        def test_un_unicode_accdtls(self):
            """ We also do testing for the un_unicode_accdtls function. """
            self.assertEqual(repseq(self.chromabsrep, un_unicode_accdtls), 
                u"C, C#, D, Eb, E, F, F#, G, Ab, A, Bb, B");

        def test_seq_dict(self):
            """ Fits all testing for the seq_dict class in here. """
            self.assertTrue(self.seq_maps.check_nseqby_subdict(NSEQ_CHORD));
            self.assertTrue(self.seq_maps.check_nseqby_subdict(NSEQ_SCALE));
            self.assertFalse(self.seq_maps.check_nseqby_subdict(666));
            self.assertTrue(self.seq_maps.check_nseqby_name(NSEQ_CHORD, "Major"));
            self.assertTrue(self.seq_maps.check_nseqby_name(NSEQ_SCALE, "Major"));
            self.assertFalse(self.seq_maps.check_nseqby_name(NSEQ_CHORD, "Minor"));
            self.assertFalse(self.seq_maps.check_nseqby_name(NSEQ_SCALE, "Minor"));
            self.assertTrue(self.seq_maps.check_nseqby_abbrv(NSEQ_CHORD, 
                "maj"));
            self.assertFalse(self.seq_maps.check_nseqby_abbrv(NSEQ_SCALE, 
                "maj"));
            self.assertFalse(self.seq_maps.check_nseqby_abbrv(NSEQ_CHORD, 
                "min"));
            self.assertFalse(self.seq_maps.check_nseqby_abbrv(NSEQ_SCALE, 
                "min"));
            self.assertFalse(self.seq_maps.check_nseqby_seqpos(NSEQ_CHORD, 
                CHROM_NAT_NOTE_POS));
            self.assertTrue(self.seq_maps.check_nseqby_seqpos(NSEQ_SCALE, 
                CHROM_NAT_NOTE_POS));
            self.assertTrue(self.seq_maps.check_nseqby_seqpos(NSEQ_CHORD,
                self.major_interval));
            self.assertFalse(self.seq_maps.check_nseqby_seqpos(NSEQ_SCALE,
                self.major_interval));
            self.assertEqual(self.seq_maps.get_nseqby_name("Major", NSEQ_CHORD), 
                self.majorchord);
            self.assertEqual(self.seq_maps.get_nseqby_name("Major", NSEQ_SCALE), 
                self.majorscale);
            self.assertEqual(self.seq_maps.get_nseqby_abbrv("maj", NSEQ_CHORD), 
                self.majorchord);                
            self.assertEqual(self.seq_maps.get_nseqby_seqpos(
                self.major_interval, NSEQ_CHORD), self.majorchord);
            self.assertEqual(self.seq_maps.get_nseqby_seqpos(
                CHROM_NAT_NOTE_POS, NSEQ_SCALE), self.majorscale);

        def test_temperament_dict(self):
            """ Like test_seq_dict, but checks dictionary in WestTemp. """
            self.assertTrue(WestTemp.check_nseqby_subdict(NSEQ_CHORD));
            self.assertTrue(WestTemp.check_nseqby_subdict(NSEQ_SCALE));
            self.assertFalse(WestTemp.check_nseqby_subdict(666));
            WestTemp.add_elem(self.majorchord, NSEQ_CHORD, "Major", "maj", 
                self.major_interval);
            WestTemp.add_elem(self.majorscale, NSEQ_SCALE, "Major", [], 
                CHROM_NAT_NOTE_POS);     
            self.assertTrue(WestTemp.check_nseqby_name(NSEQ_CHORD, "Major"));
            self.assertTrue(WestTemp.check_nseqby_name(NSEQ_SCALE, "Major"));
            self.assertFalse(WestTemp.check_nseqby_name(NSEQ_CHORD, "Minor"));
            self.assertFalse(WestTemp.check_nseqby_name(NSEQ_SCALE, "Minor"));
            self.assertTrue(WestTemp.check_nseqby_abbrv(NSEQ_CHORD, 
                "maj"));
            self.assertFalse(WestTemp.check_nseqby_abbrv(NSEQ_SCALE, 
                "maj"));
            self.assertFalse(WestTemp.check_nseqby_abbrv(NSEQ_CHORD, 
                "min"));
            self.assertFalse(WestTemp.check_nseqby_abbrv(NSEQ_SCALE, 
                "min"));
            self.assertFalse(WestTemp.check_nseqby_seqpos(NSEQ_CHORD, 
                CHROM_NAT_NOTE_POS));
            self.assertTrue(WestTemp.check_nseqby_seqpos(NSEQ_SCALE, 
                CHROM_NAT_NOTE_POS));
            self.assertTrue(WestTemp.check_nseqby_seqpos(NSEQ_CHORD,
                self.major_interval));
            self.assertFalse(WestTemp.check_nseqby_seqpos(NSEQ_SCALE,
                self.major_interval));
            self.assertEqual(WestTemp.get_nseqby_name("Major", NSEQ_CHORD), 
                self.majorchord);
            self.assertEqual(WestTemp.get_nseqby_name("Major", NSEQ_SCALE), 
                self.majorscale);
            self.assertEqual(WestTemp.get_nseqby_abbrv("maj", NSEQ_CHORD), 
                self.majorchord);                
            self.assertEqual(WestTemp.get_nseqby_seqpos(
                self.major_interval, NSEQ_CHORD), self.majorchord);
            self.assertEqual(WestTemp.get_nseqby_seqpos(
                CHROM_NAT_NOTE_POS, NSEQ_SCALE), self.majorscale);

    unittest.main()
   

    
    

